using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

using JigLibX.Physics;
using JigLibX.Geometry;
using JigLibX.Collision;

namespace Collision_1
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;


        List<SphereActor> sphere_list = new List<SphereActor>();
        Random random = new Random();
        BoxActor immovableBox, groundLevel, building, building2;
        SphereActor sphere;
        List<BoxActor> box_list = new List<BoxActor>();
        private Matrix _view;

        #region Camera
        Vector3 cameraPosition = new Vector3(38, 65, -442);
        float leftrightRot = MathHelper.PiOver2;
        float updownRot = -MathHelper.Pi / 10.0f;
        const float rotationSpeed = 0.3f;
        const float moveSpeed = 150.0f;
        #endregion

        MouseState originalMouseState;

        public Matrix View
        {
            get
            {
                return _view;
            }
        }
        public float rotation = .3f;

        private Matrix _projection;
        public Matrix Projection
        {
            get
            {
                return _projection;
            }
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            InitializePhysics();
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            _projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), (float)graphics.PreferredBackBufferWidth / (float)graphics.PreferredBackBufferHeight, 0.1f, 1000.0f);
            base.Initialize();
        }
        private void InitializePhysics()
        {
            PhysicsSystem world = new PhysicsSystem();
            world.CollisionSystem = new CollisionSystemSAP();


            immovableBox = new BoxActor(this, new Vector3(0, -5, 0), new Vector3(500, 5, 500), 0.0f, 0.0f, rotation);
            groundLevel = new BoxActor(this, new Vector3(0, -250 * (rotation), 0), new Vector3(500, 5, 500), 0.0f, 0.0f, 0.0f);
            building = new BoxActor(this, new Vector3(50, 50, 0), new Vector3(70, 355, 70), 0.0f, 0.0f, 0.0f);
            
            building2 = new BoxActor(this, new Vector3(-131, 70, -116), new Vector3(70, 355, 70), 0.0f, MathHelper.ToRadians(127), 0.0f);

            sphere = new SphereActor(this, new Vector3(220, 80, 0), 1f, 0f);

            immovableBox.Body.Immovable = true;
            groundLevel.Body.Immovable = true;
            building.Body.Immovable = true;
            building2.Body.Immovable = true;
            Components.Add(building);
            Components.Add(building2);
            Components.Add(sphere);
            Components.Add(groundLevel);
            
            Components.Add(immovableBox);
        }
        private void UpdateViewMatrix()
        {
            Matrix cameraRotation = Matrix.CreateRotationX(updownRot) * Matrix.CreateRotationY(leftrightRot);

            Vector3 cameraOriginalTarget = new Vector3(0, 0, -1);
            Vector3 cameraRotatedTarget = Vector3.Transform(cameraOriginalTarget, cameraRotation);
            Vector3 cameraFinalTarget = cameraPosition + cameraRotatedTarget;

            Vector3 cameraOriginalUpVector = new Vector3(0, 1, 0);
            Vector3 cameraRotatedUpVector = Vector3.Transform(cameraOriginalUpVector, cameraRotation);

            _view = Matrix.CreateLookAt(cameraPosition, cameraFinalTarget, cameraRotatedUpVector);
            Console.WriteLine(cameraPosition.ToString());
        }
        
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
        }
        private void ProcessInput(float amount)
        {
            MouseState currentMouseState = Mouse.GetState();
            if (currentMouseState != originalMouseState)
            {
                float xDifference = currentMouseState.X - originalMouseState.X;
                float yDifference = currentMouseState.Y - originalMouseState.Y;
                leftrightRot -= rotationSpeed * xDifference * amount;
                updownRot -= rotationSpeed * yDifference * amount;
                Mouse.SetPosition(GraphicsDevice.Viewport.Width / 2, GraphicsDevice.Viewport.Height / 2);
                UpdateViewMatrix();
            }
            Vector3 moveVector = new Vector3(0, 0, 0);
            KeyboardState keyState = Keyboard.GetState();
            if (keyState.IsKeyDown(Keys.Up) || keyState.IsKeyDown(Keys.W))
                moveVector += new Vector3(0, 0, -1);
            if (keyState.IsKeyDown(Keys.Down) || keyState.IsKeyDown(Keys.S))
                moveVector += new Vector3(0, 0, 1);
            if (keyState.IsKeyDown(Keys.Right) || keyState.IsKeyDown(Keys.D))
                moveVector += new Vector3(1, 0, 0);
            if (keyState.IsKeyDown(Keys.Left) || keyState.IsKeyDown(Keys.A))
                moveVector += new Vector3(-1, 0, 0);
            if (keyState.IsKeyDown(Keys.Q))
                moveVector += new Vector3(0, 1, 0);
            if (keyState.IsKeyDown(Keys.Z))
                moveVector += new Vector3(0, -1, 0);
            if(keyState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            AddToCameraPosition(moveVector * amount);
        }
        private void AddToCameraPosition(Vector3 vectorToAdd)
        {
            Matrix cameraRotation = Matrix.CreateRotationX(updownRot) * Matrix.CreateRotationY(leftrightRot);
            Vector3 rotatedVector = Vector3.Transform(vectorToAdd, cameraRotation);
            cameraPosition += moveSpeed * rotatedVector;
            UpdateViewMatrix();
        }
        

        protected override void UnloadContent()
        {
            
        }

        protected override void Update(GameTime gameTime)
        {


            float timeStep = (float)gameTime.ElapsedGameTime.Ticks / TimeSpan.TicksPerSecond;
            PhysicsSystem.CurrentPhysicsSystem.Integrate(timeStep);
            if (sphere_list.Count < 400)
            {
                SphereActor sph = new SphereActor(this, new Vector3(220, 80, random.Next(-40, 40)), 5f, 0f);
                sph.Body.ApplyGravity=true;
                
                sphere_list.Add(sph);
                /*if (sphere_list.Count > 320)
                {
                    StreamWriter writer = new StreamWriter(sphere_list.Count.ToString());
                    writer.WriteLine("1");
                    writer.WriteLine(sph.Body.Position.X);
                    writer.WriteLine(sph.Body.Position.Y);
                    writer.WriteLine(sph.Body.Position.Z);
                    writer.Close();
                }*/
                Components.Add(sph);
                
            }
            for (int i = 0; i < sphere_list.Count; i++) 
            {
                if(sphere_list.ElementAt<SphereActor>(i).Body.Position.X<-250)
                {
                    sphere_list.ElementAt<SphereActor>(i).Body.Position = new Vector3(220, 80, random.Next(-40, 40));
                }
                /*StreamWriter writer_2 = new StreamWriter(i.ToString(),true);
                writer_2.WriteLine(sphere_list.ElementAt<SphereActor>(i).Body.Position.X);
                writer_2.WriteLine(sphere_list.ElementAt<SphereActor>(i).Body.Position.Y);
                writer_2.WriteLine(sphere_list.ElementAt<SphereActor>(i).Body.Position.Z);
                writer_2.Close(); */
            }

            float timeDifference = (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000.0f;
            ProcessInput(timeDifference);

            Mouse.SetPosition(GraphicsDevice.Viewport.Width / 2, GraphicsDevice.Viewport.Height / 2);
            originalMouseState = Mouse.GetState();

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
           
            base.Draw(gameTime);
        }
    }
}
