﻿#region using statements
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

#endregion

namespace sph
{
    class Water2
    {
        float h = 1;
        public Vector3 gravity = new Vector3(0, -9.8f, 0);
        float density = 1f;
        float restdensity = 0f;
        float gasconstant = 1;
        float mass = 1;

        float pressure;
        float _density_;
        public Vector3 position;

        Vector3 pressure_force;
        Vector3 viscosity_force;
        Vector3 color_field_gradient;
        float color_field_laplacian;
        Vector3 velocity;
        Vector3 surface_tension_force;
        float surface_tension_coefficientl = 2;
        Vector3 total_force;
        Vector3 aceleration;



        public void init(Vector3 pos)
        {
            position = pos;
        }


        public void UPDATE(List<Water2> fluid_particles, GameTime gameTime )
        {
            _density_ = 1;
            pressure_force = new Vector3();
            viscosity_force = new Vector3();
            color_field_gradient = new Vector3();
            color_field_laplacian = 0;

            foreach (Water2 water in fluid_particles)
            {
                foreach (Water2 neighbor in fluid_particles)
                {
                    if (neighbor.position != position)
                    {
                        Vector3 r = position - neighbor.position;
                        if (r.Length() <= h)
                        {
                            density += mass * W_poly6(r, h);
                        }
                    }
                }
            }

            foreach (Water2 water in fluid_particles)
            {
                foreach (Water2 neighbor in fluid_particles)
                {
                    if (position != neighbor.position)
                    {
                        Vector3 r = position - neighbor.position;
                        if (r.Length() <= h)
                        {
                            float density_p = _density_;
                            float density_n = neighbor._density_;

                            pressure = gasconstant * (density_p - restdensity);
                            neighbor.pressure = gasconstant * (density_n - restdensity);

                            float abs_of_r = (float)Math.Sqrt(((r.X * r.X) + (r.Y * r.Y) + (r.Z * r.Z)));

                            pressure_force += mass * (pressure + neighbor.pressure) / (2 * neighbor.density) * gradient_W_spiky(r, abs_of_r);

                            viscosity_force += mass * (neighbor.velocity - velocity) / neighbor.density * lapalcian_of_W_viscosity(abs_of_r);

                            color_field_gradient += mass / neighbor.density * gradient_W_poly6(r, abs_of_r);

                            color_field_laplacian += mass / density_n * laplacian_of_W_poly6(abs_of_r);

                        }
                    }
                }
            }

            foreach (Water2 particle in fluid_particles)
            {
                
                float gradient_length = color_field_gradient.Length();
                if (gradient_length >= 1)
                {
                    surface_tension_force = -surface_tension_coefficientl * color_field_laplacian * color_field_gradient / gradient_length;
                }
                else
                {
                    surface_tension_force = Vector3.Zero;
                }
                total_force = surface_tension_force + pressure_force + viscosity_force;
                aceleration = total_force / _density_ * (float)gameTime.ElapsedGameTime.TotalSeconds + gravity;

                velocity += velocity + aceleration *(float)gameTime.ElapsedGameTime.TotalSeconds;

                position += velocity *(float)gameTime.ElapsedGameTime.TotalSeconds;

            }
        }

        public float W_poly6(Vector3 r,float h)
        {
            float return_value = (315.0f / (64.0f * MathHelper.Pi * h * h * h * h * h * h * h * h * h)) * (h * h - (r.X * r.X + r.Y * r.Y + r.Z * r.Z)) * (h * h - (r.X * r.X + r.Y * r.Y + r.Z * r.Z)) * (h * h - (r.X * r.X + r.Y * r.Y + r.Z * r.Z));
            if (float.IsNaN(return_value))
            {
                Console.WriteLine("W_poly6 returns NaN");
            }
            return return_value;
        }
        private Vector3 gradient_W_poly6(Vector3 r, float abs_of_r)
        {
            Vector3 returnit = new Vector3();
            float h_to_the_9 = h * h * h * h * h * h * h * h * h;

            returnit = (r * (945.0f / (32.0f * MathHelper.Pi * h_to_the_9)) * ((h * h - abs_of_r * abs_of_r) * (h * h - abs_of_r * abs_of_r)));
            if (float.IsNaN(returnit.X) || float.IsNaN(returnit.Y) || float.IsNaN(returnit.Z))
            {
                Console.WriteLine("gradient W poly6 returns NaN");
            }
            return returnit;
        }
        private Vector3 gradient_W_spiky(Vector3 r, float abs_of_r)
        {
            Vector3 return_val = new Vector3();
            return_val = -r * ((45.0f / (MathHelper.Pi * h * h * h * h * h * h * abs_of_r)) * ((h - abs_of_r) * (h - abs_of_r)));
            if (float.IsNaN(return_val.X) || float.IsNaN(return_val.Y) || float.IsNaN(return_val.Z))
            {
                Console.WriteLine("gradient W spikey returns NaN");
            }
            return return_val;
        }
        private float lapalcian_of_W_viscosity(float abs_of_r)
        {
            float return_valu = ((45.0f /( MathHelper.Pi * h * h * h * h * h)) * (1 - (abs_of_r / h)));
            if (float.IsNaN(return_valu))
            {
                Console.WriteLine("lapalcian_of_W_viscosity returns NaN");
            }
            return return_valu;
        }
        private float laplacian_of_W_poly6(float abs_of_r)
        {
            float r_to_the_2 = abs_of_r * abs_of_r;
            float ret = (945.0f / (8.0f * MathHelper.Pi * h * h * h * h * h * h * h * h * h)) * ((h * h) - r_to_the_2) * (r_to_the_2 - ((3 / 4) * ((h * h) - r_to_the_2)));
            if (float.IsNaN(ret))
            {
                Console.WriteLine("lapalcian_of_W_viscosity returns NaN");
            }
            return ret;
        }
    }
}
